@@ -20,6 +20,7 @@ import shlex
 import uuid
 from hashlib import sha1
 from json import loads
+from pathlib import Path
 from subprocess import PIPE, Popen
 from tempfile import TemporaryDirectory
 
@@ -40,6 +41,12 @@ from .exceptions import MermaidError
 
 logger = logging.getLogger(__name__)
 
+# Load fullscreen CSS and JavaScript from external files
+_MODULE_DIR = Path(__file__).parent
+_FULLSCREEN_CSS = (_MODULE_DIR / "fullscreen.css").read_text(encoding="utf-8")
+_MERMAID_RUN_FULLSCREEN = (_MODULE_DIR / "fullscreen.js").read_text(encoding="utf-8")
+_MERMAID_RUN_FULLSCREEN_ZOOM = (_MODULE_DIR / "fullscreen_zoom.js").read_text(encoding="utf-8")
+
 mapname_re = re.compile(r'<map id="(.*?)"')
 _MERMAID_INIT_JS_DEFAULT = "mermaid.initialize({startOnLoad:false});"
 _MERMAID_RUN_NO_D3_ZOOM = """
@@ -132,7 +139,7 @@ class Mermaid(Directive):
             if self.content:
                 return [
                     document.reporter.warning(
-                        "Mermaid directive cannot have both content and " "a filename argument",
+                        "Mermaid directive cannot have both content and a filename argument",
                         line=self.lineno,
                     )
                 ]
@@ -146,7 +153,7 @@ class Mermaid(Directive):
             except OSError:
                 return [
                     document.reporter.warning(
-                        "External Mermaid file %r not found or reading " "it failed" % filename,
+                        "External Mermaid file %r not found or reading it failed" % filename,
                         line=self.lineno,
                     )
                 ]
@@ -267,7 +274,7 @@ def render_mm(self, code, options, _fmt, prefix="mermaid"):
         try:
             p = Popen(mm_args, shell=mermaid_cmd_shell, stdout=PIPE, stdin=PIPE, stderr=PIPE)
         except FileNotFoundError:
-            logger.warning("command %r cannot be run (needed for mermaid " "output), check the mermaid_cmd setting" % mermaid_cmd)
+            logger.warning("command %r cannot be run (needed for mermaid output), check the mermaid_cmd setting" % mermaid_cmd)
             return None, None
 
         stdout, stderr = p.communicate(str.encode(code))
@@ -275,9 +282,9 @@ def render_mm(self, code, options, _fmt, prefix="mermaid"):
             logger.info(stdout)
 
         if p.returncode != 0:
-            raise MermaidError("Mermaid exited with error:\n[stderr]\n%s\n" "[stdout]\n%s" % (stderr, stdout))
+            raise MermaidError("Mermaid exited with error:\n[stderr]\n%s\n[stdout]\n%s" % (stderr, stdout))
         if not os.path.isfile(outfn):
-            raise MermaidError("Mermaid did not produce an output file:\n[stderr]\n%s\n" "[stdout]\n%s" % (stderr, stdout))
+            raise MermaidError("Mermaid did not produce an output file:\n[stderr]\n%s\n[stdout]\n%s" % (stderr, stdout))
         return relfn, outfn
 
 
@@ -310,7 +317,7 @@ def render_mm_html(self, node, code, options, prefix="mermaid", imgcls=None, alt
 
     try:
         if _fmt not in ("png", "svg"):
-            raise MermaidError("mermaid_output_format must be one of 'raw', 'png', " "'svg', but is %r" % _fmt)
+            raise MermaidError("mermaid_output_format must be one of 'raw', 'png', 'svg', but is %r" % _fmt)
 
         fname, outfn = render_mm(self, code, options, _fmt, prefix)
     except MermaidError as exc:
@@ -365,9 +372,9 @@ def render_mm_latex(self, node, code, options, prefix="mermaid"):
             logger.info(stdout)
 
         if p.returncode != 0:
-            raise MermaidError("PdfCrop exited with error:\n[stderr]\n%s\n" "[stdout]\n%s" % (stderr, stdout))
+            raise MermaidError("PdfCrop exited with error:\n[stderr]\n%s\n[stdout]\n%s" % (stderr, stdout))
         if not os.path.isfile(outfn):
-            raise MermaidError("PdfCrop did not produce an output file:\n[stderr]\n%s\n" "[stdout]\n%s" % (stderr, stdout))
+            raise MermaidError("PdfCrop did not produce an output file:\n[stderr]\n%s\n[stdout]\n%s" % (stderr, stdout))
 
         fname = "{filename[0]}-crop{filename[1]}".format(filename=os.path.splitext(fname))
 
@@ -478,6 +485,9 @@ def install_js(
         app.add_js_file(None, body=app.config.mermaid_init_js, priority=priority, type="module")
 
     _wrote_mermaid_run = False
+    _has_zoom = False
+    _has_fullscreen = app.config.mermaid_fullscreen
+
     if app.config.mermaid_output_format == "raw":
         if app.config.d3_use_local:
             _d3_js_url = app.config.d3_use_local
@@ -488,9 +498,11 @@ def install_js(
         app.add_js_file(_d3_js_url, priority=app.config.mermaid_js_priority)
 
         if app.config.mermaid_d3_zoom:
-            _d3_js_script = _MERMAID_RUN_D3_ZOOM.format(d3_selector=".mermaid svg", d3_node_count=-1, mermaid_js_url=_mermaid_js_url)
-            app.add_js_file(None, body=_d3_js_script, priority=app.config.mermaid_js_priority, type="module")
-            _wrote_mermaid_run = True
+            _has_zoom = True
+            if not _has_fullscreen:
+                _d3_js_script = _MERMAID_RUN_D3_ZOOM.format(d3_selector=".mermaid svg", d3_node_count=-1, mermaid_js_url=_mermaid_js_url)
+                app.add_js_file(None, body=_d3_js_script, priority=app.config.mermaid_js_priority, type="module")
+                _wrote_mermaid_run = True
         elif doctree:
             mermaid_nodes = doctree.findall(mermaid)
             _d3_selector = ""
@@ -504,9 +516,53 @@ def install_js(
                         _d3_selector += f", .mermaid[data-zoom-id={_zoom_id}] svg"
                     count += 1
             if _d3_selector != "":
-                _d3_js_script = _MERMAID_RUN_D3_ZOOM.format(d3_selector=_d3_selector, d3_node_count=count, mermaid_js_url=_mermaid_js_url)
-                app.add_js_file(None, body=_d3_js_script, priority=app.config.mermaid_js_priority, type="module")
-                _wrote_mermaid_run = True
+                _has_zoom = True
+                if not _has_fullscreen:
+                    _d3_js_script = _MERMAID_RUN_D3_ZOOM.format(d3_selector=_d3_selector, d3_node_count=count, mermaid_js_url=_mermaid_js_url)
+                    app.add_js_file(None, body=_d3_js_script, priority=app.config.mermaid_js_priority, type="module")
+                    _wrote_mermaid_run = True
+
+    # Handle fullscreen feature
+    if _has_fullscreen and not _wrote_mermaid_run:
+        _button_text = app.config.mermaid_fullscreen_button
+        _button_opacity = app.config.mermaid_fullscreen_button_opacity
+        if _has_zoom:
+            # Fullscreen with zoom
+            _d3_selector = ".mermaid svg" if app.config.mermaid_d3_zoom else ""
+            if not _d3_selector and doctree:
+                # Build selector for per-diagram zoom
+                mermaid_nodes = doctree.findall(mermaid)
+                count = 0
+                for mermaid_node in mermaid_nodes:
+                    if "zoom_id" in mermaid_node:
+                        _zoom_id = mermaid_node["zoom_id"]
+                        if _d3_selector == "":
+                            _d3_selector += f".mermaid[data-zoom-id={_zoom_id}] svg"
+                        else:
+                            _d3_selector += f", .mermaid[data-zoom-id={_zoom_id}] svg"
+                        count += 1
+                if _d3_selector == "":
+                    _d3_selector = ".mermaid svg"
+                    count = -1
+            else:
+                count = -1
+            _d3_js_script = _MERMAID_RUN_FULLSCREEN_ZOOM.format(
+                mermaid_js_url=_mermaid_js_url,
+                fullscreen_css=_FULLSCREEN_CSS,
+                button_text=_button_text,
+                button_opacity=_button_opacity,
+                d3_selector=_d3_selector if _d3_selector else ".mermaid svg",
+                d3_node_count=count if _d3_selector else -1,
+            )
+            app.add_js_file(None, body=_d3_js_script, priority=app.config.mermaid_js_priority, type="module")
+            _wrote_mermaid_run = True
+        else:
+            # Fullscreen without zoom
+            _fullscreen_js_script = _MERMAID_RUN_FULLSCREEN.format(
+                mermaid_js_url=_mermaid_js_url, fullscreen_css=_FULLSCREEN_CSS, button_text=_button_text
+            )
+            app.add_js_file(None, body=_fullscreen_js_script, priority=app.config.mermaid_js_priority, type="module")
+            _wrote_mermaid_run = True
 
     if not _wrote_mermaid_run and _mermaid_js_url:
         app.add_js_file(
@@ -535,7 +591,7 @@ def setup(app):
     app.add_config_value("mermaid_sequence_config", False, "html")
 
     app.add_config_value("mermaid_use_local", "", "html")
-    app.add_config_value("mermaid_version", "11.2.0", "html")
+    app.add_config_value("mermaid_version", "11.12.1", "html")
     app.add_config_value("mermaid_elk_use_local", "", "html")
     app.add_config_value("mermaid_include_elk", "0.1.4", "html")
     app.add_config_value("mermaid_js_priority", 500, "html")
@@ -544,6 +600,9 @@ def setup(app):
     app.add_config_value("d3_use_local", "", "html")
     app.add_config_value("d3_version", "7.9.0", "html")
     app.add_config_value("mermaid_d3_zoom", False, "html")
+    app.add_config_value("mermaid_fullscreen", True, "html")
+    app.add_config_value("mermaid_fullscreen_button", "⛶", "html")
+    app.add_config_value("mermaid_fullscreen_button_opacity", "50", "html")
     app.connect("html-page-context", install_js)
 
     return {"version": sphinx.__display_version__, "parallel_read_safe": True}
